﻿using CMS;
using CMS.CustomTables;
using CMS.DataEngine;
using CMS.EventLog;
using CMS.Helpers;
using CMS.Scheduler;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;


[assembly: RegisterCustomClass("Custom.BulkImportDemoTask", typeof(Custom.BulkImportDemoTask))]

namespace Custom
{
    public class BulkImportDemoTask : ITask
    {
        /// <summary>
        /// Executes the task.
        /// </summary>
        /// <param name="ti">Info object representing the scheduled task</param>
        public string Execute(TaskInfo ti)
        {
            try
            {
                string[] taskparams = ti.TaskData.Split(',');

                bool blnCleanupData = ValidationHelper.GetBoolean(taskparams[0], false);
                bool blnOptimized = ValidationHelper.GetBoolean(taskparams[1], false);

                #region Cleanup Data
                // Determine if the demo data should be cleaned up
                if (blnCleanupData)
                {
                    GeneralConnection cnCleanUp = ConnectionHelper.GetConnection();
                    QueryParameters qpCleanUp = new QueryParameters("TRUNCATE TABLE custom_exoplanets", null, QueryTypeEnum.SQLQuery, false);
                    cnCleanUp.ExecuteNonQuery(qpCleanUp);
                }
                #endregion

                // Get import data
                // In this demo, I'm getting it from another database by specifiying a different connection string 
                GeneralConnection cnExoplanets = ConnectionHelper.GetConnectionByName("ExoplanetConnectionString");
                QueryParameters qp = new QueryParameters("SELECT * FROM Exoplanets", null, QueryTypeEnum.SQLQuery, false);
                DataSet dsImport = cnExoplanets.ExecuteQuery(qp);

                if (!blnOptimized)
                {
                    #region Inefficient Code
                    // This section uses the API to loop through each object and perform an INSERT
                    // Prepares the code name (class name) of the custom table to which the data record will be added
                    string customTableClassName = "custom.exoplanets";

                    // Gets the custom table
                    DataClassInfo customTable = DataClassInfoProvider.GetDataClassInfo(customTableClassName);
                    if (customTable != null)
                    {
                        // Loop through each record and perfrom an insert
                        foreach (DataRow dr in dsImport.Tables[0].Rows)
                        {
                            // Creates a new custom table item 
                            CustomTableItem newCustomTableItem = CustomTableItem.New(customTableClassName);

                            // Sets the values for the fields of the custom table (ItemText in this case)
                            newCustomTableItem.SetValue("PlanetName", ValidationHelper.GetString(dr["name"], ""));
                            newCustomTableItem.SetValue("StarName", ValidationHelper.GetString(dr["star_name"], ""));
                            newCustomTableItem.SetValue("YearDiscovered", ValidationHelper.GetInteger(dr["discovered"], 1900));

                            // Save the new custom table record into the database
                            newCustomTableItem.Insert();
                        }
                    };
                    #endregion
                }
                else
                {
                    #region Optimized Code
                    // This section uses the BulkInsert API and passes the collection of items in a single database call
                    // First, create a BulkInsertSettings object to hold the filed mappings for the source and destination tables
                    BulkInsertSettings settings = new BulkInsertSettings
                    {
                        Mappings = new Dictionary<string, string>
                        {
                            {"name", "PlanetName"},
                            {"star_name", "StarName"},
                            {"discovered", "YearDiscovered"}
                        }
                    };

                    // Use the BulkInsert API to insert all of the items at once.
                    ConnectionHelper.BulkInsert(dsImport.Tables[0], "custom_exoplanets", settings);
                    #endregion
                }

                return dsImport.Tables[0].Rows.Count + " exoplanets imported! (Optimized: " + blnOptimized + ")";
            }
            catch (Exception ex)
            {
                EventLogProvider.LogException("BulkImportDemoTask", "EXECUTE", ex);
                return ex.Message;
            }
        }
    }
}